
#include "bibli.h"
#include "tas.h"
#include "iris.h"
#include "knn.h"


/* 

-------------- PARTIE I : TESTS TAS ---------------

*/

void newline(){
  printf("\n");
}

void test_create(){
  printf("\n TEST_CREATE \n");
  heap * res = create(5);
  printf("res->capacity=%d; res->size=%d\n",res->capacity,res->size);
  free(res->stock);
  free(res);
}


void test_more_priority(){
  printf("\n TEST_MORE_PRIORITY \n");
  ATOMIC_COORD_TYPE t1[COORD_SIZE] = {3.,6.};
  ATOMIC_COORD_TYPE t2[COORD_SIZE] = {31.,63.};
  data d1 = {.priority=5.,.coords=t1,.info=-1};
  data d2 = {.priority=4.,.coords=t2,.info=-1};
  printf("d1.priority=%f,d2.priority=%f\n",d1.priority,d2.priority);
  printf("more_priority(d1,d2)=%d\n",more_priority(d1,d2));
  printf("more_priority(d2,d1)=%d\n",more_priority(d2,d1));
}

void test_display_data(){
  printf("\n TEST_DISPLAY_DATA \n");
  ATOMIC_COORD_TYPE t1[COORD_SIZE]={3.,6.};
  data d1 = {.priority=5.,.coords=t1,.info=-1};
  display_data(d1);
  printf("\n");
}

/*


cf 

%https://stackoverflow.com/questions/62432414/c-why-variables-created-in-a-loop-have-the-same-memory-address

pour le comportement curieux suivant :

Curieusement, les tableaux créés dans la boucle for
ont tous la même adresse !!

  for (int i=0; i<6; i++){
    ATOMIC_COORD_TYPE t[COORD_SIZE] = {(float) i*2, (float) i * 3};
    printf("adresse t=%p\n",&t[0]);

On obtient toujours la même adresse, WTF !!
*/
void test_add(){
  printf("\n TEST_ADD \n");
  heap * h = create(3);
  printf("h->size=%d,h->capacity=%d\n",h->size,h->capacity);
  int final_size_of_heap = 6;
  //on crée un tableau de 6 tuples de COORD_SIZE coords
  ATOMIC_COORD_TYPE t[final_size_of_heap][COORD_SIZE];;
  for (int i=0; i<6; i++){
    t[i][0] = (float) i * 2;
    t[i][1] = (float) i * 3;
    t[i][2] = 0.;
    t[i][3] = 0.;
    data d = {.priority=i,.coords=t[i],.info=-1};
    add(h,d);
    printf("h->size=%d,h->capacity=%d\n",h->size,h->capacity);
    printf("\n");
  }
  printf("contenu du tas:\n");
  display_heap(h);

  printf("\n");
  free(h->stock);
  free(h);
}

void test_remove(){
  printf("\n TEST_REMOVE \n");
  heap * h = create(3);
  int final_size_of_heap = 6;
  //on crée un tableau de 6 tuples de COORD_SIZE coords
  ATOMIC_COORD_TYPE t[final_size_of_heap][COORD_SIZE];;
  for (int i=0; i<6; i++){
    t[i][0] = (float) i * 2;
    t[i][1] = (float) i * 3;
    t[i][2] = 0.;
    t[i][3] = 0.;
    data d = {.priority=i,.coords=t[i],.info=-1};
    add(h,d);
  }//for
  display_heap(h);
  
  printf("\nremove_node(h,1)=");
  display_data(remove_node(h,1));newline();
  printf("contenu du tas:\n");display_heap(h);
  printf("\nremove_node(h,3)=");
  display_data(remove_node(h,3));newline();
  printf("contenu du tas:\n");
  display_heap(h);
  
  printf("\n");
  free(h->stock);
  free(h);
}

/*

 PARTIE II

 ---------- TESTS IRIS ------------

*/

void test_display_variety(){
  printf("\n TEST_DISPLAY_VARIETY \n");
  enum variety v = SETOSA;
  display_variety(v);newline();
  v = VERSICOLOUR;
  display_variety(v);newline();
  v = VIRGINICA;
  display_variety(v);newline();
  data d = {0., coords[100],categories[100]};
  display_data(d);newline();
}

/*

 PARTIE III

-------------- TESTS KNN ------------------

*/

//copier p2 dans p1
void _copy_coord(COORD_TYPE p1, COORD_TYPE p2){
  for (int i=0;i<COORD_SIZE;i++)
    p1[i]=p2[i];
}

void test_dist(){
  printf("\nTEST_DIST \n");
  ATOMIC_COORD_TYPE p1[COORD_SIZE]={};
  ATOMIC_COORD_TYPE p2[COORD_SIZE]={1.,2.,2.,0.};
  printf("point p1 : ");display_float_arr(p1);newline();
  printf("point p2 : ");display_float_arr(p2);newline();
  printf("dist(p1,p2)=%f",dist(p1,p2));newline();
}

void test_knn(){
  printf("\nTEST_KNN \n");
  //point coords[134] ou Donnes de tests 189 : 
  ATOMIC_COORD_TYPE p[COORD_SIZE]={6.1, 2.6, 5.6, 1.4};
  printf("point p=");display_float_arr(p);newline();
  ATOMIC_COORD_TYPE coords_apprentissage[DATA_NUMBER/3][COORD_SIZE];
  enum variety categories_apprentissage[DATA_NUMBER/3];
  for (int i=0;i<DATA_NUMBER;i++){
    if (i%3==0){
      _copy_coord(coords_apprentissage[i/3],coords[i]);
      categories_apprentissage[i/3]=categories[i];
    }//i%3==0
  }
  printf("Fin construction données d'apprentissage\n");
  /* AFFICHAGE donnees apprentissage 
  for (int i=0; i<DATA_NUMBER/3;i++){
    display_float_arr(coords_apprentissage[i]);
    newline();
    display_float_arr(coords[i*3]);
    newline();
    display_variety(categories_apprentissage[i]);newline();
    display_variety(categories[i*3]);
    newline();
  }
  printf("Fin affichage données d'apprentissage\n");
  */
  heap * h = knn(DATA_NUMBER/3,
		 coords_apprentissage,
		 categories_apprentissage,
		 6,
		 p);
  printf("Fin construction du tas\n");
  display_heap(h);
  free(h->stock);
  free(h);
}



void test_attribution(){
  printf("\nTEST_ATTRIBUTION \n");
  //point coords[134] ou Donnes de tests 189 : 
  ATOMIC_COORD_TYPE p[COORD_SIZE]={6.1, 2.6, 5.6, 1.4};
  printf("point p=");display_float_arr(p);newline();
  ATOMIC_COORD_TYPE coords_apprentissage[DATA_NUMBER/3][COORD_SIZE];
  enum variety categories_apprentissage[DATA_NUMBER/3];
  for (int i=0;i<DATA_NUMBER;i++){
    if (i%3==0){
      _copy_coord(coords_apprentissage[i/3],coords[i]);
      categories_apprentissage[i/3]=categories[i];
    }//i%3==0
  }
  printf("Fin construction données d'apprentissage\n");
  printf("attribution d'une classe à p\n");
  enum variety m = attribution(DATA_NUMBER/3,
		 coords_apprentissage,
		 categories_apprentissage,
		 6,
		 p);
  display_variety(m);newline();
  // o doit obtenir virginica
}


int main(){
  test_create();
  test_more_priority();
  test_display_data();
  test_add();
  test_remove();
  test_display_variety();
  test_dist();
  test_knn();
  test_attribution();
  return EXIT_SUCCESS;
}
